Package: consort
Type: Package
Title: Create Consort Diagram
Version: 1.2.3
Authors@R: person("Alim", "Dayim",
                  email = "ad938@cam.ac.uk",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0001-9998-7463"))
Description: To make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved.
License: MIT + file LICENSE
URL: https://github.com/adayim/consort/
BugReports: https://github.com/adayim/consort/issues
Encoding: UTF-8
Imports: grid
Suggests: testthat (>= 3.0.0), knitr, DiagrammeR, rmarkdown, covr,
        stringi
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Depends: R (>= 3.5.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-25 21:37:19 UTC; alim
Author: Alim Dayim [aut, cre] (ORCID: <https://orcid.org/0000-0001-9998-7463>)
Maintainer: Alim Dayim <ad938@cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-26 02:10:02 UTC
