# Generated by roxygen2: do not edit by hand

S3method(heightDetails,textbox)
S3method(makeContent,textbox)
S3method(makeContext,textbox)
S3method(plot,consort)
S3method(print,consort)
S3method(widthDetails,textbox)
S3method(xDetails,textbox)
S3method(yDetails,textbox)
export(add_box)
export(add_label_box)
export(add_side_box)
export(add_split)
export(build_grid)
export(build_grviz)
export(connect_box)
export(consort_plot)
export(gen_text)
export(get_coords)
export(grid.textbox)
export(move_box)
export(textbox)
import(grid)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,modifyList)
