#' Create Consort diagram
#'
#' To make it easy to create CONSORT diagrams for the transparent reporting
#'  of participant allocation in randomized, controlled clinical trials. This
#'   is done by creating a standardized disposition data, and using this data
#'    as the source for the creation a standard CONSORT diagram. Human effort
#'     by supplying text labels on the node can also be achieved.
#'     
#' @docType package
#' @name consort
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import grid
#' @importFrom utils modifyList
## usethis namespace: end
NULL
